@@ -3,7 +3,7 @@
 # Copyright (C) 2013 Nippon Telegraph and Telephone Corporation.
 # Copyright (C) 2015 Brad Cowie, Christopher Lorier and Joe Stringer.
 # Copyright (C) 2015 Research and Education Advanced Network New Zealand Ltd.
-# Copyright (C) 2015--2018 The Contributors
+# Copyright (C) 2015--2019 The Contributors
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -35,6 +35,9 @@ class FaucetMetrics(PromClient):
         self.PORT_REQUIRED_LABELS = self.REQUIRED_LABELS + ['port', 'port_description']
         self._dpid_counters = {}
         self._dpid_gauges = {}
+        self.faucet_stack_root_dpid = self._gauge(
+            'faucet_stack_root_dpid',
+            'set to current stack root DPID', [])
         self.faucet_config_reload_requests = self._counter(
             'faucet_config_reload_requests',
             'number of config reload requests', [])
@@ -64,7 +67,7 @@ class FaucetMetrics(PromClient):
             'number of OF packet_ins received but ignored from DP (due to rate limiting)')
         self.of_unexpected_packet_ins = self._dpid_counter(
             'of_unexpected_packet_ins',
-            'number of OF packet_ins received that are unexpected from DP (e.g. for VLAN that is not configured)')
+            'number of OF packet_ins received that are unexpected from DP (e.g. for unknown VLAN)')
         self.of_packet_ins = self._dpid_counter(
             'of_packet_ins',
             'number of OF packet_ins received from DP')
@@ -141,13 +144,20 @@ class FaucetMetrics(PromClient):
             'port_learn_bans',
             'number of times learning was banned on a port',
             self.PORT_REQUIRED_LABELS)
-        self.port_lacp_status = self._gauge(
-            'port_lacp_status',
-            'status of LACP on port',
+        self.learned_l2_port = self._gauge(
+            'learned_l2_port',
+            'learned port of l2 entries',
+            self.REQUIRED_LABELS + ['vid', 'eth_src'])
+        self.port_lacp_state = self._gauge(
+            'port_lacp_state',
+            'state of LACP on a port',
             self.PORT_REQUIRED_LABELS)
         self.dp_status = self._dpid_gauge(
             'dp_status',
             'status of datapaths')
+        self.dp_root_hop_port = self._dpid_gauge(
+            'dp_root_hop_port',
+            'port that leads to stack root DP')
         self.of_dp_desc_stats = self._gauge(
             'of_dp_desc_stats',
             'DP description (OFPDescStatsReply)',
