@@ -2,7 +2,7 @@
 
 # Copyright (C) 2015 Brad Cowie, Christopher Lorier and Joe Stringer.
 # Copyright (C) 2015 Research and Education Advanced Network New Zealand Ltd.
-# Copyright (C) 2015--2018 The Contributors
+# Copyright (C) 2015--2019 The Contributors
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -23,10 +23,16 @@ import netaddr
 
 from faucet import valve_of
 from faucet.conf import Conf, test_config_condition, InvalidConfigError
-from faucet.faucet_pipeline import STACK_LOOP_PROTECT_FIELD
 from faucet.valve_packet import FAUCET_MAC
 
 
+class OFVLAN:
+
+    def __init__(self, name, vid):
+        self.name = name
+        self.vid = vid
+
+
 class NullVLAN:
     """Placeholder null VLAN."""
 
@@ -87,6 +93,8 @@ class VLAN(Conf):
         'description': None,
         'acl_in': None,
         'acls_in': None,
+        'acl_out': None,
+        'acls_out': None,
         'faucet_vips': None,
         'faucet_mac': FAUCET_MAC,
         # set MAC for FAUCET VIPs on this VLAN
@@ -105,8 +113,10 @@ class VLAN(Conf):
         # If False, don't check that IP packets have a payload (OVS trace/tutorial requires False).
         'reserved_internal_vlan': False,
         # If True, forward packets from the VLAN table to the VLAN_ACL table matching the VID
-        'dot1x_assigned' : False,
-        # If True, this VLAN may be dynamically added by using the Tunnel-Private-Group-ID radius attribute
+        'dot1x_assigned': False,
+        # If True, this VLAN may be dynamically added withTunnel-Private-Group-ID radius attribute.
+        'edge_learn_stack_root': False,
+        # If True, this VLAN will learn flows through the stack root, following forwarding path.
         }
 
     defaults_types = {
@@ -114,6 +124,8 @@ class VLAN(Conf):
         'description': str,
         'acl_in': (int, str),
         'acls_in': list,
+        'acl_out': (int, str),
+        'acls_out': list,
         'faucet_vips': list,
         'faucet_mac': str,
         'unicast_flood': bool,
@@ -126,15 +138,19 @@ class VLAN(Conf):
         'minimum_ip_size_check': bool,
         'reserved_internal_vlan': bool,
         'dot1x_assigned': bool,
+        'edge_learn_stack_root': bool,
     }
 
     def __init__(self, _id, dp_id, conf=None):
         self.acl_in = None
         self.acls_in = None
+        self.acl_out = None
+        self.acls_out = None
         self.description = None
         self.dot1x_assigned = None
         self.dot1x_untagged = None
         self.dp_id = None
+        self.edge_learn_stack_root = None
         self.faucet_mac = None
         self.faucet_vips = None
         self.max_hosts = None
@@ -185,13 +201,22 @@ class VLAN(Conf):
 
         test_config_condition(
             self.acl_in and self.acls_in, 'found both acl_in and acls_in, use only acls_in')
+        test_config_condition(
+            self.acl_out and self.acls_out, 'found both acl_out and acls_out, use only acls_out')
         if self.acl_in and not isinstance(self.acl_in, list):
             self.acls_in = [self.acl_in,]
             self.acl_in = None
+        if self.acl_out and not isinstance(self.acl_out, list):
+            self.acls_out = [self.acl_out,]
+            self.acl_out = None
+        all_acls = []
         if self.acls_in:
-            for acl in self.acls_in:
-                test_config_condition(
-                    not isinstance(acl, (int, str)), 'acl names must be int or str')
+            all_acls.extend(self.acls_in)
+        if self.acls_out:
+            all_acls.extend(self.acls_out)
+        for acl in all_acls:
+            test_config_condition(
+                not isinstance(acl, (int, str)), 'acl names must be int or str')
 
         if self.max_hosts:
             if not self.proactive_arp_limit:
@@ -203,6 +228,10 @@ class VLAN(Conf):
             self.faucet_vips = frozenset([
                 self._check_ip_str(ip_str, ip_method=ipaddress.ip_interface)
                 for ip_str in self.faucet_vips])
+            for faucet_vip in self.faucet_vips:
+                test_config_condition(
+                    faucet_vip.network.prefixlen == faucet_vip.max_prefixlen,
+                    'VIP cannot be a host address')
 
         if self.routes:
             test_config_condition(not isinstance(self.routes, list), 'invalid VLAN routes format')
@@ -226,9 +255,7 @@ class VLAN(Conf):
     @staticmethod
     def vid_valid(vid):
         """Return True if VID valid."""
-        if isinstance(vid, int) and vid >= valve_of.MIN_VID and vid <= valve_of.MAX_VID:
-            return True
-        return False
+        return isinstance(vid, int) and vid >= valve_of.MIN_VID and vid <= valve_of.MAX_VID
 
     def reset_caches(self):
         """Reset dynamic caches."""
@@ -355,6 +382,7 @@ class VLAN(Conf):
         return False
 
     def _update_gw_types(self, ip_gw):
+        """Update dyn host/route gw information to a different ip version"""
         if self.is_host_fib_route(ip_gw):
             self.dyn_host_gws_by_ipv[ip_gw.version].add(ip_gw)
             self.dyn_route_gws_by_ipv[ip_gw.version] -= set([ip_gw])
@@ -418,6 +446,10 @@ class VLAN(Conf):
         """Return all ports on this VLAN."""
         return self.tagged + self.untagged + self.dot1x_untagged
 
+    def restricted_bcast_arpnd_ports(self):
+        """Return all ports with restricted broadcast enabled."""
+        return tuple([port for port in self.get_ports() if port.restricted_bcast_arpnd])
+
     def hairpin_ports(self):
         """Return all ports with hairpin enabled."""
         return tuple([port for port in self.get_ports() if port.hairpin])
@@ -438,9 +470,10 @@ class VLAN(Conf):
         """Return ports that have LACP on this VLAN."""
         return tuple([port for port in self.get_ports() if port.lacp])
 
-    def lacp_up_ports(self):
-        """Return ports that have LACP up on this VLAN."""
-        return tuple([port for port in self.lacp_ports() if port.dyn_lacp_up])
+    def lacp_up_selected_ports(self):
+        """Return LACP ports that have been SELECTED and are UP"""
+        return tuple([
+            port for port in self.lacp_ports() if port.is_port_selected() and port.is_actor_up()])
 
     def lags(self):
         """Return dict of LAGs mapped to member ports."""
@@ -449,30 +482,29 @@ class VLAN(Conf):
             lags[port.lacp].append(port)
         return lags
 
-    def lags_up(self):
-        """Return dict of LAGs mapped to member ports that have LACP up."""
+    def selected_up_lags(self):
+        """Return dict of LAGs mapped to member ports that have been selected"""
         lags = collections.defaultdict(list)
-        for port in self.lacp_up_ports():
+        for port in self.lacp_up_selected_ports():
             lags[port.lacp].append(port)
         return lags
 
-    def exclude_same_lag_member_ports(self, in_port=None):
-        """Ensure output on only one member of a LAG."""
+    def excluded_lag_ports(self, in_port=None):
+        """Ensure output to SELECTED LAG ports & only one LAG member"""
         exclude_ports = set()
         lags = self.lags()
         if lags:
-            lags_up = self.lags_up()
+            # Need lags that have actor UP & are SELECTED
+            selected_ports = self.selected_up_lags()
             if in_port is not None and in_port.lacp:
-                # Don't flood from one LACP bundle member, to another.
+                # Don't flood to same LAG
                 exclude_ports.update(lags[in_port.lacp])
-            # Pick one up bundle member to flood to.
+            # Pick a bundle member to flood to
             for lag, ports in lags.items():
-                ports_up = lags_up[lag]
-                if ports_up:
-                    ports.remove(ports_up[0])
-                    exclude_ports.update(ports)
-                else:
-                    exclude_ports.update(ports)
+                selected_lag = selected_ports[lag]
+                if selected_lag:
+                    ports.remove(selected_lag[0])
+                exclude_ports.update(ports)
         return exclude_ports
 
     def exclude_native_if_dot1x(self):
@@ -487,6 +519,7 @@ class VLAN(Conf):
 
     @staticmethod
     def flood_ports(configured_ports, exclude_unicast):
+        """Return configured ports that allow flooding"""
         if exclude_unicast:
             return tuple([port for port in configured_ports if port.unicast_flood])
         return configured_ports
@@ -497,7 +530,7 @@ class VLAN(Conf):
     def untagged_flood_ports(self, exclude_unicast):
         return self.flood_ports(self.untagged + self.dot1x_untagged, exclude_unicast)
 
-    def output_port(self, port, hairpin=False, output_table=None, loop_protect_field=None):
+    def output_port(self, port, hairpin=False, output_table=None, external_forwarding_requested=None):
         actions = []
         if self.port_is_untagged(port):
             actions.append(valve_of.pop_vlan())
@@ -505,8 +538,11 @@ class VLAN(Conf):
             actions.extend(port.mirror_actions())
         else:
             actions.extend(port.mirror_actions())
-            if loop_protect_field is not None:
-                actions.append(output_table.set_field(**{STACK_LOOP_PROTECT_FIELD: loop_protect_field}))
+            if external_forwarding_requested is not None:
+                if external_forwarding_requested:
+                    actions.append(output_table.set_external_forwarding_requested())
+                else:
+                    actions.append(output_table.set_no_external_forwarding_requested())
         if hairpin:
             actions.append(valve_of.output_port(valve_of.OFP_IN_PORT))
         else:
@@ -514,6 +550,7 @@ class VLAN(Conf):
         return actions
 
     def pkt_out_port(self, packet_builder, port, *args):
+        """Return packet-out actions with VLAN tag if port is tagged"""
         vid = None
         if self.port_is_tagged(port):
             vid = self.vid
@@ -521,13 +558,14 @@ class VLAN(Conf):
         return valve_of.packetout(port.number, pkt.data)
 
     def flood_pkt(self, packet_builder, multi_out=True, *args):
+        """Return Packet-out actions via flooding"""
         ofmsgs = []
         for vid, ports in (
                 (self.vid, self.tagged_flood_ports(False)),
                 (None, self.untagged_flood_ports(False))):
             if ports:
                 pkt = packet_builder(vid, *args)
-                exclude_ports = self.exclude_same_lag_member_ports()
+                exclude_ports = self.excluded_lag_ports()
                 running_port_nos = [
                     port.number for port in ports if port.running() and port not in exclude_ports]
                 if running_port_nos:
@@ -548,6 +586,7 @@ class VLAN(Conf):
         return port in self.untagged or port in self.dot1x_untagged
 
     def vip_map(self, ipa):
+        """Return the vip containing ipa"""
         for faucet_vip in self.faucet_vips:
             if ipa in faucet_vip.network:
                 return faucet_vip
