@@ -3,12 +3,12 @@
 FAUCETHOME=`dirname $0`"/../.."
 PYTHONPATH=$FAUCETHOME:$FAUCETHOME/clib
 
-MINRATING=9.4
+MINRATING=9.44
 
 lintfile=`tempfile`.lint
 
 for f in $* ; do
-    PYTHONPATH=$PYTHONPATH pylint --rcfile=/dev/null --extension-pkg-whitelist=netifaces,pytricia $f > $lintfile
+    PYTHONPATH=$PYTHONPATH pylint --rcfile=/dev/null --extension-pkg-whitelist=netifaces,pytricia -d import-error $f > $lintfile
     rating=`cat $lintfile | grep -ohE "rated at [0-9\.]+" | sed "s/rated at //g"`
     echo pylint $f: $rating
     failing=$(bc <<< "$rating < $MINRATING")
