@@ -1,5 +1,5 @@
 # Copyright (C) 2015 Research and Innovation Advanced Network New Zealand Ltd.
-# Copyright (C) 2015--2018 The Contributors
+# Copyright (C) 2015--2019 The Contributors
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -13,27 +13,37 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
+import sys
+import argparse
+import json
+import ast
 from bitstring import Bits
 from ryu.ofproto import ofproto_v1_3 as ofp
 from ryu.ofproto import ofproto_v1_3_parser as parser
+from ryu.ofproto import ofproto_parser as ofp_parser
 from ryu.lib import addrconv
 
 
-class FakeOFTableException(Exception):
+CONTROLLER_PORT = 4294967293
 
-    pass
+
+class FakeOFTableException(Exception):
+    """Indicates an erroneous flow or group mod"""
 
 
 class FakeOFTable:
-    """Fake OFTable is a virtual openflow pipeline used for testing openflow controllers.
+    """Fake OFTable is a virtual openflow pipeline used for testing openflow
+    controllers.
 
     The tables are populated using apply_ofmsgs and can be queried with
-    is_ouput.
+    is_output.
     """
 
-    def __init__(self, num_tables):
+    def __init__(self, num_tables=1, requires_tfm=True):
         self.tables = [[] for _ in range(0, num_tables)]
         self.groups = {}
+        self.requires_tfm = requires_tfm
+        self.tfm = {}
 
     def _apply_groupmod(self, ofmsg):
         """Maintain group table."""
@@ -47,12 +57,14 @@ class FakeOFTable:
 
         def _add(ofmsg, group_id):
             if group_id in self.groups:
-                raise FakeOFTableException('group already in group table: %s' % ofmsg)
+                raise FakeOFTableException(
+                    'group already in group table: %s' % ofmsg)
             self.groups[group_id] = ofmsg
 
         def _modify(ofmsg, group_id):
             if group_id not in self.groups:
-                raise FakeOFTableException('group not in group table: %s' % ofmsg)
+                raise FakeOFTableException(
+                    'group not in group table: %s' % ofmsg)
             self.groups[group_id] = ofmsg
 
         _groupmod_handlers = {
@@ -67,6 +79,24 @@ class FakeOFTable:
         """Adds, Deletes and modify flow modification messages are applied
            according to section 6.4 of the OpenFlow 1.3 specification."""
 
+        def _validate_flowmod_tfm(table_id, tfm_body, ofmsg):
+            if not self.requires_tfm:
+                return
+
+            if table_id == ofp.OFPTT_ALL:
+                if ofmsg.match.items() and not self.tfm:
+                    raise FakeOFTableException(
+                        'got %s with matches before TFM that defines tables'
+                        % ofmsg)
+                return
+
+            if tfm_body is None:
+                raise FakeOFTableException(
+                    'got %s before TFM that defines table %u' % (
+                        ofmsg, table_id
+                        )
+                    )
+
         def _add(table, flowmod):
             # From the 1.3 spec, section 6.4:
             # For add requests (OFPFC_ADD) with the
@@ -90,7 +120,7 @@ class FakeOFTable:
                 if flowmod.fte_matches(fte, strict=True):
                     table.remove(fte)
                     break
-                elif flowmod.overlaps(fte):
+                if flowmod.overlaps(fte):
                     raise FakeOFTableException(
                         'Overlapping flowmods {} and {}'.format(
                             flowmod, fte))
@@ -127,15 +157,44 @@ class FakeOFTable:
         }
 
         table_id = ofmsg.table_id
+        tfm_body = self.tfm.get(table_id, None)
+
         if table_id == ofp.OFPTT_ALL or table_id is None:
             tables = self.tables
         else:
             tables = [self.tables[table_id]]
+
+        _validate_flowmod_tfm(table_id, tfm_body, ofmsg)
         flowmod = FlowMod(ofmsg)
 
         for table in tables:
             _flowmod_handlers[ofmsg.command](table, flowmod)
 
+        if tfm_body:
+            for table in tables:
+                entries = len(table)
+                if entries > tfm_body.max_entries:
+                    tfm_table_details = 'table %u %s full (max %u)' % (
+                        table_id, tfm_body.name, tfm_body.max_entries)
+                    flow_dump = '\n\n'.join(
+                        (tfm_table_details, str(ofmsg), str(tfm_body)))
+                    raise FakeOFTableException(flow_dump)
+
+    def _apply_tfm(self, ofmsg):
+        self.tfm = {body.table_id: body for body in ofmsg.body}
+
+    def _apply_flowstats(self, ofmsg):
+        """Update state of flow tables to match an OFPFlowStatsReply message.
+
+        This assumes a tfm is not required."""
+        self.tables = []
+        self.requires_tfm = False
+        self.tfm = {}
+        for stat in ofmsg.body:
+            while len(self.tables) <= stat.table_id:
+                self.tables.append([])
+            self.tables[stat.table_id].append(FlowMod(stat))
+
     def apply_ofmsgs(self, ofmsgs):
         """Update state of test flow tables."""
         for ofmsg in ofmsgs:
@@ -149,12 +208,12 @@ class FakeOFTable:
                 continue
             if isinstance(ofmsg, parser.OFPDescStatsRequest):
                 continue
-            if isinstance(ofmsg, parser.OFPTableFeaturesStatsRequest):
-                # TODO: validate TFM
-                continue
             if isinstance(ofmsg, parser.OFPMeterMod):
                 # TODO: handle OFPMeterMod
                 continue
+            if isinstance(ofmsg, parser.OFPTableFeaturesStatsRequest):
+                self._apply_tfm(ofmsg)
+                continue
             if isinstance(ofmsg, parser.OFPGroupMod):
                 self._apply_groupmod(ofmsg)
                 continue
@@ -162,9 +221,13 @@ class FakeOFTable:
                 self._apply_flowmod(ofmsg)
                 self.sort_tables()
                 continue
+            if isinstance(ofmsg, parser.OFPFlowStatsReply):
+                self._apply_flowstats(ofmsg)
+                self.sort_tables()
+                continue
             raise FakeOFTableException('Unsupported flow %s' % str(ofmsg))
 
-    def lookup(self, match):
+    def lookup(self, match, trace=False):
         """Return the entries from flowmods that matches match.
 
         Searches each table in the pipeline for the entries that will be
@@ -178,7 +241,7 @@ class FakeOFTable:
         Returns: a list of the flowmods that will be applied to the packet
                 represented by match
         """
-        packet_dict = match.copy() # Packet headers may be modified
+        packet_dict = match.copy()  # Packet headers may be modified
         instructions = []
         table_id = 0
         goto_table = True
@@ -193,6 +256,8 @@ class FakeOFTable:
                     break
             # if a flowmod is found, make modifications to the match values and
             # determine if another lookup is necessary
+            if trace:
+                sys.stderr.write('%d: %s\n' % (table_id, matching_fte))
             if matching_fte:
                 for instruction in matching_fte.instructions:
                     instructions.append(instruction)
@@ -212,7 +277,11 @@ class FakeOFTable:
                             | (instruction.metadata & mask)
         return (instructions, packet_dict)
 
-    def is_output(self, match, port=None, vid=None):
+    def flow_count(self):
+        """Return number of flow tables rules"""
+        return sum(map(len, self.tables))
+
+    def is_output(self, match, port=None, vid=None, trace=False):
         """Return true if packets with match fields is output to port with
         correct vlan.
 
@@ -222,8 +291,8 @@ class FakeOFTable:
         If vid is none it will return true if output to specified port
         regardless of vlan tag.
 
-        To specify the packet should be output without a vlan tag set the
-        OFPVID_PRESENT bit in vid is 0.
+        To specify checking that the packet should not have a vlan tag, set the
+        OFPVID_PRESENT bit in vid to 0.
 
         Arguments:
         Match: a dictionary keyed by header field names with values.
@@ -231,13 +300,20 @@ class FakeOFTable:
         def _output_result(action, vid_stack, port, vid):
             if port is None:
                 return True
+            in_port = match.get('in_port')
+            result = None
             if action.port == port:
-                if vid is None:
-                    return True
-                if vid & ofp.OFPVID_PRESENT == 0:
-                    return not vid_stack
-                return vid_stack and vid == vid_stack[-1]
-            return None
+                if port == in_port:
+                    result = None
+                elif vid is None:
+                    result = True
+                elif vid & ofp.OFPVID_PRESENT == 0:
+                    result = not vid_stack
+                else:
+                    result = vid_stack and vid == vid_stack[-1]
+            elif action.port == ofp.OFPP_IN_PORT and port == in_port:
+                result = True
+            return result
 
         def _process_vid_stack(action, vid_stack):
             if action.type == ofp.OFPAT_PUSH_VLAN:
@@ -249,13 +325,17 @@ class FakeOFTable:
                     vid_stack[-1] = action.value
             return vid_stack
 
+        if trace:
+            sys.stderr.write('tracing packet flow %s\n' % match)
+            sys.stderr.write(str(self) + '\n')
+
         # vid_stack represents the packet's vlan stack, innermost label listed
         # first
         match_vid = match.get('vlan_vid', 0)
         vid_stack = []
         if match_vid & ofp.OFPVID_PRESENT != 0:
             vid_stack.append(match_vid)
-        instructions, _ = self.lookup(match)
+        instructions, _ = self.lookup(match, trace=trace)
 
         for instruction in instructions:
             if instruction.type != ofp.OFPIT_APPLY_ACTIONS:
@@ -263,7 +343,8 @@ class FakeOFTable:
             for action in instruction.actions:
                 vid_stack = _process_vid_stack(action, vid_stack)
                 if action.type == ofp.OFPAT_OUTPUT:
-                    output_result = _output_result(action, vid_stack, port, vid)
+                    output_result = _output_result(
+                        action, vid_stack, port, vid)
                     if output_result is not None:
                         return output_result
                 elif action.type == ofp.OFPAT_GROUP:
@@ -287,10 +368,11 @@ class FakeOFTable:
         """
         Send packet through the fake OF table pipeline
         Args:
-            match (dict): A dict keyed by header fields with values, represents a packet
+            match (dict): A dict keyed by header fields with values, represents
+                a packet
         Returns:
-            dict: Modified match dict, represents packet that has been through the pipeline \
-                with values possibly altered
+            dict: Modified match dict, represents packet that has been through
+                the pipeline with values possibly altered
         """
         _, packet_dict = self.lookup(match)
         return packet_dict
@@ -298,8 +380,8 @@ class FakeOFTable:
     def __str__(self):
         string = ''
         for table_id, table in enumerate(self.tables):
-            string += '----- Table %u -----\n' % (table_id)
-            string += '\n'.join([str(flowmod) for flowmod in table])
+            string += '\n----- Table %u -----\n' % (table_id)
+            string += '\n'.join(sorted([str(flowmod) for flowmod in table]))
         return string
 
     def sort_tables(self):
@@ -317,18 +399,22 @@ class FlowMod:
         )
     IPV4_MATCH_FIELDS = ('ipv4_src', 'ipv4_dst', 'arp_spa', 'arp_tpa')
     IPV6_MATCH_FIELDS = ('ipv6_src', 'ipv6_dst', 'ipv6_nd_target')
+    HEX_FIELDS = ('eth_type')
 
     def __init__(self, flowmod):
         """flowmod is a ryu flow modification message object"""
         self.priority = flowmod.priority
+        self.cookie = flowmod.cookie
         self.instructions = flowmod.instructions
         self.validate_instructions()
         self.match_values = {}
         self.match_masks = {}
         self.out_port = None
-        if ((flowmod.command == ofp.OFPFC_DELETE or flowmod.command == ofp.OFPFC_DELETE_STRICT) and
-                flowmod.out_port != ofp.OFPP_ANY):
-            self.out_port = flowmod.out_port
+        # flowmod can be an OFPFlowMod or an OFPStats
+        if isinstance(flowmod, parser.OFPFlowMod):
+            if flowmod.command in (ofp.OFPFC_DELETE, ofp.OFPFC_DELETE_STRICT)\
+                    and flowmod.out_port != ofp.OFPP_ANY:
+                self.out_port = flowmod.out_port
 
         for key, val in flowmod.match.items():
             if isinstance(val, tuple):
@@ -446,6 +532,19 @@ class FlowMod:
             return _val_to_bits(addrconv.ipv6.text_to_bin, val, 128)
         return Bits(int=int(val), length=64)
 
+    def bits_to_str(self, key, val):
+        if key in self.MAC_MATCH_FIELDS:
+            result = addrconv.mac.bin_to_text(val.tobytes())
+        elif key in self.IPV4_MATCH_FIELDS:
+            result = addrconv.ipv4.bin_to_text(val.tobytes())
+        elif key in self.IPV6_MATCH_FIELDS:
+            result = addrconv.ipv6.bin_to_text(val.tobytes())
+        elif key in self.HEX_FIELDS:
+            result = str(val.hex.lstrip('0'))
+        else:
+            result = str(val.int)
+        return result
+
     def __lt__(self, other):
         return self.priority < other.priority
 
@@ -454,12 +553,226 @@ class FlowMod:
                 self.out_port == other.out_port and
                 self.instructions == other.instructions)
 
+    def _pretty_field_str(self, key, value, mask=None):
+        mask_str = ""
+        value_int = value
+        mask_int = mask
+        if isinstance(value, Bits):
+            value_int = value.int
+        if isinstance(mask, Bits):
+            mask_int = mask.int  # pytype: disable=attribute-error
+        elif mask is None:
+            mask_int = -1
+        if key == 'vlan_vid':
+            if value_int & ofp.OFPVID_PRESENT == 0:
+                result = 'vlan untagged'
+            elif key == 'vlan_vid' and mask_int == ofp.OFPVID_PRESENT:
+                result = 'vlan tagged'
+            else:
+                result = str(value_int ^ ofp.OFPVID_PRESENT)
+                if mask_int != -1:
+                    mask_str = str(mask_int ^ ofp.OFPVID_PRESENT)
+        elif isinstance(value, Bits):
+            result = self.bits_to_str(key, value)
+            if mask is not None and mask_int != -1:
+                mask_str = self.bits_to_str(key, mask)
+        elif isinstance(value, str):
+            result = value
+            if mask is not None:
+                mask_str = mask
+        elif isinstance(value, int):
+            if key in self.HEX_FIELDS:
+                result = hex(value)
+                if mask is not None and mask != -1:
+                    mask_str = hex(mask)
+            else:
+                result = str(value)
+                if mask is not None and mask != -1:
+                    mask_str = str(mask)
+        if mask_str:
+            result += "/{}".format(mask_str)
+        return result
+
+    def _pretty_action_str(self, action):
+        actions_names_attrs = {
+            parser.OFPActionPushVlan.__name__: ('push_vlan', 'ethertype'),
+            parser.OFPActionPopVlan.__name__: ('pop_vlan', None),
+            parser.OFPActionGroup.__name__: ('group', 'group_id'),
+            }
+        value = None
+        if isinstance(action, parser.OFPActionOutput):
+            name = 'output'
+            if action.port == CONTROLLER_PORT:
+                value = 'controller'
+            else:
+                value = str(action.port)
+        elif isinstance(action, parser.OFPActionSetField):
+            name = 'set_{}'.format(action.key)
+            value = self._pretty_field_str(action.key, action.value)
+        else:
+            name, attr = actions_names_attrs[type(action).__name__]
+            if attr:
+                value = getattr(action, attr)
+        result = name
+        if value:
+            result += " {}".format(value)
+        return result
+
     def __str__(self):
-        string = 'priority: {0}'.format(self.priority)
-        for key, val in self.match_values.items():
+        result = 'Priority: {0} | Match: '.format(self.priority)
+
+        for key in sorted(self.match_values.keys()):
+            val = self.match_values[key]
             mask = self.match_masks[key]
-            string += ' {0}: {1}'.format(key, val)
-            if mask.int != -1: # pytype: disable=attribute-error
+            result += " {} {},".format(
+                key, self._pretty_field_str(key, val, mask))
+        result = result.rstrip(',')
+        result += " | Instructions :"
+        if not self.instructions:
+            result += ' drop'
+        for instruction in self.instructions:
+            if isinstance(instruction, parser.OFPInstructionGotoTable):
+                result += ' goto {}'.format(instruction.table_id)
+            elif isinstance(instruction, parser.OFPInstructionActions):
+                for action in instruction.actions:
+                    result += " {},".format(self._pretty_action_str(action))
+        result = result.rstrip(',')
+        return result
+
+    def __repr__(self):
+        string = 'priority: {0} cookie: {1}'.format(self.priority, self.cookie)
+        for key in sorted(self.match_values.keys()):
+            mask = self.match_masks[key]
+            string += ' {0}: {1}'.format(key, self.match_values[key])
+            if mask.int != -1:  # pytype: disable=attribute-error
                 string += '/{0}'.format(mask)
         string += ' Instructions: {0}'.format(str(self.instructions))
         return string
+
+
+class FakeRyuDp:  # pylint: disable=too-few-public-methods
+    """Fake ryu Datapath object.
+
+    Just needed to provide a parser to allow us to extract ryu objects from
+    JSON
+    """
+    def __init__(self):
+        self.ofproto_parser = parser
+
+
+def parse_print_args():
+    """Parse arguments for the print command"""
+    arg_parser = argparse.ArgumentParser(
+        prog='fakeoftable',
+        description='Prints a JSON flow table in a human readable format',
+        usage="""
+    Print a flow table in a human readable format
+    {argv0} print -f FILE
+""".format(argv0=sys.argv[0])
+        )
+    arg_parser.add_argument(
+        '-f',
+        '--file',
+        help='file containing an OFPFlowStatsReply message in JSON format'
+        )
+    args = arg_parser.parse_args(sys.argv[2:])
+    return {'filename': args.file}
+
+
+def parse_probe_args():
+    """Parse arguments for the probe command"""
+    arg_parser = argparse.ArgumentParser(
+        prog='fakeoftable',
+        description='Performs a packet lookup on a JSON openflow table',
+        usage="""
+    Find the flow table entries in a given flow table that match a given packet
+    {argv0} probe -f FILE -p PACKET_STRING
+""".format(argv0=sys.argv[0])
+        )
+    arg_parser.add_argument(
+        '-p',
+        '--packet',
+        metavar='PACKET_STRING',
+        help=(
+            '''string representation of a packet dictionary eg. '''
+            '''"{'in_port': 1, 'eth_dst': '01:80:c2:00:00:02', 'eth_type': '''
+            '''34825}"''')
+        )
+    arg_parser.add_argument(
+        '-f',
+        '--file',
+        metavar='FILE',
+        help='file containing an OFPFlowStatsReply message in JSON format'
+        )
+    args = arg_parser.parse_args(sys.argv[2:])
+    packet = args.packet
+    packet = ast.literal_eval(args.packet)
+    # fix vlan vid
+    if 'vlan_vid' in packet:
+        packet['vlan_vid'] |= ofp.OFPVID_PRESENT
+    return {'packet': packet, 'filename': args.file}
+
+
+def parse_args():
+    """parse arguments"""
+    arg_parser = argparse.ArgumentParser(
+        prog='fakeoftable',
+        description='Performs operations on JSON openflow tables',
+        usage="""
+    {argv0} <command> <args>
+
+""".format(argv0=sys.argv[0])
+        )
+    arg_parser.add_argument(
+        'command',
+        help='Subcommand, either "print" or "probe"'
+        )
+    args = arg_parser.parse_args(sys.argv[1:2])
+    try:
+        if args.command == 'probe':
+            command_args = parse_probe_args()
+        elif args.command == 'print':
+            command_args = parse_print_args()
+    except (KeyError, IndexError, ValueError, AttributeError) as err:
+        print(err)
+        arg_parser.print_help()
+        sys.exit(-1)
+    return (args.command, command_args)
+
+
+def _print(filename):
+    """Prints the JSON flow table from a file in a human readable format"""
+    with open(filename, 'r') as f:
+        msg = json.load(f)
+    dp = FakeRyuDp()
+    ofmsg = ofp_parser.ofp_msg_from_jsondict(dp, msg)
+    table = FakeOFTable()
+    table.apply_ofmsgs([ofmsg])
+    print(table)
+
+
+def probe(filename, packet):
+    """Prints the actions applied to packet by the table from the file"""
+    with open(filename, 'r') as f:
+        msg = json.load(f)
+    dp = FakeRyuDp()
+    ofmsg = ofp_parser.ofp_msg_from_jsondict(dp, msg)
+    table = FakeOFTable()
+    table.apply_ofmsgs([ofmsg])
+    instructions, out_packet = table.lookup(packet)
+    print(packet)
+    for instruction in instructions:
+        print(instruction)
+    print(out_packet)
+
+
+def main():
+    command, kwargs = parse_args()
+    if command == 'probe':
+        probe(**kwargs)
+    elif command == 'print':
+        _print(**kwargs)
+
+
+if __name__ == '__main__':
+    main()
