@@ -309,6 +309,83 @@ dps:
         self.assertTrue(port.is_actor_up(), 'Actor not UP')
 
 
+class ValveStackMCLAGStandbyTestCase(ValveTestBases.ValveTestSmall):
+    """Test MCLAG with standby port option overrules unselected states"""
+
+    CONFIG = """
+dps:
+    s1:
+%s
+        stack:
+            priority: 1
+        interfaces:
+            1:
+                description: p1
+                stack:
+                    dp: s2
+                    port: 1
+            2:
+                description: p3
+                native_vlan: 100
+                lacp_standby: True
+                lacp: 1
+            3:
+                description: p4
+                native_vlan: 100
+                lacp_standby: True
+                lacp: 1
+    s2:
+        hardware: 'GenericTFM'
+        dp_id: 0x2
+        interfaces:
+            1:
+                description: p1
+                stack:
+                    dp: s1
+                    port: 1
+            2:
+                description: p3
+                native_vlan: 100
+                lacp_standby: True
+                lacp: 1
+            3:
+                description: p4
+                native_vlan: 100
+                lacp_standby: True
+                lacp: 1
+""" % BASE_DP1_CONFIG
+
+    def setUp(self):
+        """Setup basic loop config"""
+        self.setup_valve(self.CONFIG)
+
+    def get_other_valves(self, valve):
+        """Return other running valves"""
+        return self.valves_manager._other_running_valves(valve)  # pylint: disable=protected-access
+
+    def test_MCLAG_standby_option(self):
+        """Test MCLAG standby option forces standby state instead of unselected"""
+        self.activate_all_ports()
+        valve = self.valves_manager.valves[0x1]
+        other_valve = self.valves_manager.valves[0x2]
+        for port in valve.dp.ports.values():
+            if port.lacp:
+                valve.lacp_update(port, True, 1, 1, self.get_other_valves(valve))
+                self.assertTrue(port.is_port_selected())
+        for port in other_valve.dp.ports.values():
+            if port.lacp:
+                other_valve.lacp_update(port, True, 1, 1, self.get_other_valves(other_valve))
+                self.assertTrue(port.is_port_standby())
+        for port in valve.dp.ports.values():
+            if port.lacp:
+                valve.lacp_update(port, False, 1, 1, self.get_other_valves(valve))
+                self.assertTrue(port.is_port_standby())
+        for port in other_valve.dp.ports.values():
+            if port.lacp:
+                other_valve.lacp_update(port, True, 1, 1, self.get_other_valves(other_valve))
+                self.assertTrue(port.is_port_selected())
+
+
 class ValveStackRootExtLoopProtectTestCase(ValveTestBases.ValveTestSmall):
     """External loop protect test cases"""
 
@@ -2155,5 +2232,75 @@ dps:
         self.assertTrue(dp_obj.is_stack_edge())
 
 
+class GroupDeleteACLTestCase(ValveTestBases.ValveTestSmall):
+    """Test that a group ACL creates a groupdel for the group_id"""
+
+    CONFIG = """
+acls:
+    group-acl:
+        - rule:
+            dl_dst: "0e:00:00:00:02:02"
+            actions:
+                output:
+                    failover:
+                        group_id: 1001
+                        ports: [2, 3]
+vlans:
+    vlan100:
+        vid: 100
+dps:
+    s1:
+        dp_id: 0x1
+        hardware: 'GenericTFM'
+        interfaces:
+            1:
+                native_vlan: vlan100
+                acls_in: [group-acl]
+            2:
+                native_vlan: vlan100
+            3:
+                native_vlan: vlan100
+"""
+
+    def setUp(self):
+        self.setup_valve(self.CONFIG)
+
+    def check_groupmods_exist(self, ofmsgs, groupdel_exists=True):
+        """Test that the ACL groupmods exist when expected"""
+        groupdel = None
+        groupmod = None
+        for ofmsg in ofmsgs:
+            if valve_of.is_groupdel(ofmsg) and not valve_of.is_global_groupdel(ofmsg):
+                groupdel = ofmsg
+            elif valve_of.is_groupmod(ofmsg):
+                groupmod = ofmsg
+        self.assertIsNotNone(groupmod)
+        if groupdel_exists:
+            self.assertIsNotNone(groupdel)
+            self.assertTrue(groupdel.group_id, 1001)
+        else:
+            self.assertIsNone(groupdel)
+
+    def test_groupdel_exists(self):
+        """Test valve_flowreorder doesn't remove groupmods unless expected"""
+        valve = self.valves_manager.valves[0x1]
+        port = valve.dp.ports[1]
+        ofmsgs = valve.acl_manager.add_port(port)
+        self.check_groupmods_exist(valve_of.valve_flowreorder(ofmsgs))
+        global_flowmod = valve_of.flowmod(
+            0, ofp.OFPFC_DELETE, ofp.OFPTT_ALL,
+            0, ofp.OFPP_CONTROLLER, ofp.OFPP_CONTROLLER,
+            valve_of.match_from_dict({}), [], 0, 0, 0)
+        self.check_groupmods_exist(
+            valve_of.valve_flowreorder(ofmsgs + [global_flowmod]))
+        global_metermod = valve_of.meterdel()
+        self.check_groupmods_exist(
+            valve_of.valve_flowreorder(ofmsgs + [global_flowmod, global_metermod]))
+        global_groupmod = valve_of.groupdel()
+        self.check_groupmods_exist(
+            valve_of.valve_flowreorder(
+                ofmsgs + [global_flowmod, global_metermod, global_groupmod]), False)
+
+
 if __name__ == "__main__":
     unittest.main()  # pytype: disable=module-attr
