@@ -45,8 +45,10 @@ if [ "${MATRIX_SHARD}" == "unittest" ] ; then
   exit 0
 elif [ "${MATRIX_SHARD}" == "sanity" ] ; then
   FAUCET_TESTS="-u FaucetSanityTest"
+elif [ "${MATRIX_SHARD}" == "fault-tolerance" ] ; then
+  FAUCET_TESTS="-t"
 else
-  ALLTESTFILES="tests/integration/mininet_tests.py clib/clib_mininet_tests.py"
+  ALLTESTFILES="tests/integration/mininet_tests.py tests/integration/mininet_multidp_tests.py clib/clib_mininet_tests.py"
   ALLTESTS=`grep -E -o "^class (Faucet[a-zA-Z0-9]+Test)" ${ALLTESTFILES}|cut -f2 -d" "|sort`
   declare -A sharded
 
@@ -62,7 +64,7 @@ else
   }
 
   shard "$ALLTESTS" ${MATRIX_SHARDS}
-  FAUCET_TESTS="-di ${sharded[${MATRIX_SHARD}]}"
+  FAUCET_TESTS="-din ${sharded[${MATRIX_SHARD}]}"
 fi
 
 if [[ "$FILES_CHANGED" != "" ]] ; then
@@ -74,23 +76,28 @@ if [[ "$FILES_CHANGED" != "" ]] ; then
   fi
 fi
 
-docker pull faucet/test-base
-docker build -t ${FAUCET_TEST_IMG} -f Dockerfile.tests . || exit 1
+docker build --pull -t ${FAUCET_TEST_IMG} -f Dockerfile.tests . || exit 1
 docker rmi faucet/test-base
 docker images
 
-echo Shard $MATRIX_SHARD: $FAUCETTESTS
-ulimit -c unlimited && echo '/var/tmp/core.%h.%e.%t' > /proc/sys/kernel/core_pattern
-sudo modprobe openvswitch
-# TODO: --network=host for https://www.traviscistatus.com/incidents/kyf149kl6bvp?u=3g7kt113nmgs
-sudo docker run --privileged --sysctl net.ipv6.conf.all.disable_ipv6=0 \
-  --network=host \
+SHARDARGS="--privileged --sysctl net.ipv6.conf.all.disable_ipv6=0 \
   --ulimit core=99999999999:99999999999 \
   -v /var/local/lib/docker:/var/lib/docker \
-  -v $HOME/.cache/pip:/var/tmp/pip-cache \
-  -e FAUCET_TESTS="${FAUCET_TESTS}" \
-  -e PY_FILES_CHANGED="${PY_FILES_CHANGED}" \
-  -t ${FAUCET_TEST_IMG} || exit 1
+  -v $HOME/.cache/pip:/var/tmp/pip-cache"
+echo Shard $MATRIX_SHARD: $FAUCETTESTS: $SHARDARGS
+
+ulimit -c unlimited && sudo echo '/var/tmp/core.%h.%e.%t' > /proc/sys/kernel/core_pattern
+sudo modprobe openvswitch
+sudo modprobe ebtables
+
+if [ "${MATRIX_SHARD}" == "sanity" ] ; then
+  # Simulate hardware test switch
+  # TODO: run a standalone DP and also a stacked DP test to test hardware linkages.
+  sudo docker run $SHARDARGS -e FAUCET_TESTS="-ni FaucetSanityTest FaucetStackStringOfDPUntaggedTest" -e HWTESTS="1" -t ${FAUCET_TEST_IMG} || exit 1
+fi
+
+sudo docker run $SHARDARGS -e PY_FILES_CHANGED="${PY_FILES_CHANGED}" -e FAUCET_TESTS="${FAUCET_TESTS}" -t ${FAUCET_TEST_IMG} || exit 1
+
 if ls -1 /var/tmp/core* >/dev/null 2>&1 ; then
   echo coredumps found after tests run.
   exit 1
